#include <stdio.h>
#include <stdlib.h>
#include <csfort.h>

/* NOTE:  The structures CMC_complex and CMC_double_complex are defined in csfort.h
	  and look like the following:

typedef struct 
	{
	float real;
	float imag;
	}
	  CMC_complex;

typedef struct 
	{
	double real;
	double imag;
	}
	  CMC_double_complex;
*/

void
CM_make_vec_shape ( shape_ptr, size )
  shape *shape_ptr;
  int size;
/*************************************************************
 Function:	CM_make_vec_shape
 By:		Shawn Carpenter
 Date:		9/16/93

  This function allocates CM parallel variable shapes.  It
  does for vectors (1-d) shapes only.  It is passed a pointer
  to an unallocated shape, and returns the same  pointer
  to an allocated shape.  Note that size MUST be in bools.
 **************************************************************/
{
  *shape_ptr = allocate_shape( shape_ptr, 1, size );
  if (shape_ptr == NULL) {
	printf("Not enough memory on parallel processor -- terminating\n");
	exit(1);
  }

}


void
CM_make_mat_shape ( shape_ptr, msize, nsize )
  shape *shape_ptr;
  int nsize, msize;
/*************************************************************
 Function:	CM_make_mat_shape
 By:		Shawn Carpenter
 Date:		9/16/93

  This function allocates CM parallel variable shapes.  It
  does for matrix (2-d) shapes only.  It is passed a pointer
  to an unallocated shape, and returns the allocated shape
  on the pointer.  Returned shape is msize x nsize.
 **************************************************************/
{
  *shape_ptr = allocate_shape( shape_ptr, 2, msize, nsize );
  if (shape_ptr == NULL) {
	printf("Not enough memory on parallel processor -- terminating\n");
	exit(1);
  }

}


void
CM_free_shape(shape_ptr)
  shape *shape_ptr;
/*************************************************************
 Function:	CM_free_shape
 By:		Shawn Carpenter
 Date:		9/16/93

  This function deallocates a CM parallel variable shape.
  Takes a pointer to a shape, and deallocates the shape def.
 **************************************************************/
{

  if (shape_ptr != NULL) deallocate_shape( shape_ptr );

}


void
CM_free_d_pvar( pv )
  double:void *pv;
/*************************************************************
 Function:	CM_free_d_pvar
 By:		Shawn Carpenter
 Date:		9/22/93

  Free the memory allocated to a double precision parallel 
    variable.
 **************************************************************/
{
  if (pv != NULL) pfree( pv );

}



void
CM_allocate_d_pvar( pv, vshape )
  double:void *pv;
  shape *vshape;
/*************************************************************
 Function:	CM_allocate_d_pvar
 By:		Shawn Carpenter
 Date:		9/22/93

  Allocate a double precision parallel variable.
  This function allocates CM parallel variables.  It
  does so for vectors (1-d) shapes only.  It is passed a pointer
  to an allocated shape, and allocates parallel memory for the 
  parallel vector pv.
 **************************************************************/
{
  if ( vshape == NULL ) {
    printf("Error in CM_allocate_d_pvar: Shape is not defined -- terminating\n");
    exit(1);
  }

  pv = palloc( *vshape, boolsizeof(double) );

  if (pv == NULL) {
    printf("Not enough memory on the parallel processor -- terminating\n");
    exit(1);
  }

}

void
CM_allocate_f_pvar( pv, vshape )
  double:void *pv;
  shape *vshape;
/*************************************************************
 Function:	CM_allocate_f_pvar
 By:		Shawn Carpenter
 Date:		9/22/93

  Allocate a single precision (real) parallel variable.
  This function allocates CM parallel variables.  It
  does so for vectors (1-d) shapes only.  It is passed a pointer
  to an allocated shape, and allocates parallel memory for the 
  parallel vector pv.
 **************************************************************/
{
  if ( vshape == NULL ) {
    printf("Error in CM_allocate_f_pvar: Shape is not defined -- terminating\n");
    exit(1);
  }

  pv = palloc( *vshape, boolsizeof(float) );

  if (pv == NULL) {
    printf("Not enough memory on the parallel processor -- terminating\n");

    exit(1);
  }

}
  

void
CM_allocate_z_pvar( pv, vshape )
  double:void *pv;
  shape *vshape;
/*************************************************************
 Function:	CM_allocate_z_vector
 By:		Shawn Carpenter
 Date:		9/22/93

  Allocate a double complex parallel pvar (CMC_double_complex).
  This function allocates CM parallel variables.  It
  does so for vectors (1-d) shapes only.  It is passed a pointer
  to an allocated shape, and allocates parallel memory for the 
  parallel vector pv.
 **************************************************************/
{
  if ( vshape == NULL ) {
    printf("Error in CM_allocate_z_pvar: Shape is not defined -- terminating\n");
    exit(1);
  }

  pv = palloc( *vshape, boolsizeof(CMC_double_complex) );

  if (pv == NULL) {
    printf("Not enough memory on the parallel processor -- terminating\n");
    exit(1);
  }

}


void
CM_allocate_c_pvar( pv, vshape )
  double:void *pv;
  shape *vshape;
/*************************************************************
 Function:	CM_allocate_c_pvar
 By:		Shawn Carpenter
 Date:		9/22/93

  Allocate a parallel variable of type complex (CMC_complex).
  This function allocates CM parallel variables.  It
  does so for vectors (1-d) shapes only.  It is passed a pointer
  to an allocated shape, and allocates parallel memory for the 
  parallel vector pv.
 **************************************************************/
{
  if ( vshape == NULL ) {
    printf("Error in CM_allocate_c_pvar: Shape is not defined -- terminating\n");
    exit(1);
  }

  pv = palloc( *vshape, boolsizeof(CMC_complex) );

  if (pv == NULL) {
    printf("Not enough memory on the parallel processor -- terminating\n");
    exit(1);
  }

}



void
CM_D_LU_SOLVE(m,n,o,r,a,v)
 int m,n,o,r;
 double:void *a,*v;
/*************************************************************
 Function:	CM_D_LU_SOLVE
 By:		Shawn Carpenter
 Date:		9/21/93

  This function provides a solution to a double precision
  (REAL) matrix equation, Ax=v, where x is returned in v.
  v is a matrix with multiple right-hand side vectors to be
  applied for solutions.
  Arguments:  A : pointer to parallel (m x n) matrix of double
	      v : pointer to parallel (n x o) matrix of double 
		  with r right-hand side vectors requiring solutions
  CALLS:  cm_d_lu_solve(), a CMF subroutine located in the 
            CMF module, CM_futil.fcm.
 **************************************************************/
{

  CMC_descriptor_t a_desc, v_desc;

  a_desc = CMC_wrap_pvar(a);
  v_desc = CMC_wrap_pvar(v);

  CMC_CALL_FORTRAN(cm_d_lu_solve_(&m,&n,&o,&r,a_desc,v_desc));

  CMC_free_desc(a_desc);
  CMC_free_desc(v_desc);

} /* END CM_D_LU_SOLVE */ 


CM_F_LU_SOLVE(m,n,o,r,a,v)
 int m,n,o,r;
 float:void *a,*v;
/*************************************************************
 Function:	CM_F_LU_SOLVE
 By:		Shawn Carpenter
 Date:		9/21/93

  This function provides a solution to a single precision
  (REAL) matrix equation, Ax=v, where x is returned in v.
  v is a matrix with multiple right-hand side vectors to be
  applied for solutions.
  Arguments:  A : pointer to parallel (m x n) matrix of float
	      v : pointer to parallel (n x o) matrix of float 
		  with r right-hand side vectors requiring 
		  solutions
  CALLS:  cm_f_lu_solve(), a CMF subroutine located in the 
            CMF module, CM_futil.fcm.
 **************************************************************/
{

  CMC_descriptor_t a_desc, v_desc;

  a_desc = CMC_wrap_pvar(a);
  v_desc = CMC_wrap_pvar(v);

  CMC_CALL_FORTRAN(cm_f_lu_solve_(&m,&n,&o,&r,a_desc,v_desc));

  CMC_free_desc(a_desc);
  CMC_free_desc(v_desc);

} /* END CM_F_LU_SOLVE */ 


CM_C_LU_SOLVE(m,n,o,r,a,v)
 int m,n,o,r;
 CMC_complex:void *a,*v;
/*************************************************************
 Function:	CM_C_LU_SOLVE
 By:		Shawn Carpenter
 Date:		9/21/93

  This function provides a solution to a single precision
  (COMPLEX) matrix equation, Ax=v, where x is returned in v.
  v is a matrix with multiple right-hand side vectors to be
  applied for solutions.
  Arguments:  A : pointer to parallel (m x n) matrix of complex 
		  (CMC_complex)
	      v : pointer to parallel (n x o) matrix of complex 
		  with r right-hand side vectors requiring 
		  solutions
  CALLS:  cm_c_lu_solve(), a CMF subroutine located in the 
            CMF module, CM_futil.fcm.
 **************************************************************/
{

  CMC_descriptor_t a_desc, v_desc;

  a_desc = CMC_wrap_pvar(a);
  v_desc = CMC_wrap_pvar(v);

  CMC_CALL_FORTRAN(cm_c_lu_solve_(&m,&n,&o,&r,a_desc,v_desc));

  CMC_free_desc(a_desc);
  CMC_free_desc(v_desc);

} /* END CM_C_LU_SOLVE */ 


CM_Z_LU_SOLVE(m,n,o,r,a,v)
 int m,n,o,r;
 CMC_double_complex:void *a,*v;
/*************************************************************
 Function:	CM_Z_LU_SOLVE
 By:		Shawn Carpenter
 Date:		9/21/93

  This function provides a solution to a double precision
  (COMPLEX) matrix equation, Ax=v, where x is returned in v.
  v is a matrix with multiple right-hand side vectors to be
  applied for solutions.
  Arguments:  A : pointer to parallel (m x n) matrix of double 
		  complex (CMC_double_complex)
	      v : pointer to parallel (n x o) matrix of double 
		  complex with r right-hand side vectors 
		  requiring solutions
  CALLS:  cm_z_lu_solve(), a CMF subroutine located in the 
            CMF module, CM_futil.fcm.
 **************************************************************/
{

  CMC_descriptor_t a_desc, v_desc;

  a_desc = CMC_wrap_pvar(a);
  v_desc = CMC_wrap_pvar(v);

  CMC_CALL_FORTRAN(cm_z_lu_solve_(&m,&n,&o,&r,a_desc,v_desc));

  CMC_free_desc(a_desc);
  CMC_free_desc(v_desc);

} /* END CM_Z_LU_SOLVE */ 


void
CM_Z_FFT(pv)
CMC_double_complex:void *pv;
/*************************************************************
 Function:	CM_Z_FFT
 By:		Shawn Carpenter
 Date:		9/22/93

  Double precision complex FFT (C* data type CMC_double_complex)
  1-dimension *or* 2-dimension.
  This function performs a forward 2D FFT on the dataset in the
  parallel variable pv.  The shape and size of the parallel
  variable need not be determined here; the CMSSL library routine
  determines the setup from the shape of the parallel variable.
  If the variable is 1D (Shape []S), then a 1D FFT is performed.
  If the variable is 2D (Shape [][]S), then a 2D FFT is performed.
  Transformed data is returned in pv (in-place operation).

  CALLS:  cm_z_fft(), a CMF subroutine located in the 
            CMF module, CM_futil.fcm.
 **************************************************************/
{
  
  CMC_descriptor_t cmplx_desc;

      
  cmplx_desc = CMC_wrap_pvar(pv);
    
  CMC_CALL_FORTRAN(cm_z_fft_(cmplx_desc));

  CMC_free_desc(cmplx_desc);

}

void
CM_C_FFT(pv)
CMC_complex:void *pv;
/*************************************************************
 Function:	CM_C_FFT
 By:		Shawn Carpenter
 Date:		9/22/93

  Single precision complex FFT (C* data type CMC_complex)
  1-dimension *or* 2-dimension.
  This function performs a forward 2D FFT on the dataset in the
  parallel variable pv.  The shape and size of the parallel
  variable need not be determined here; the CMSSL library routine
  determines the setup from the shape of the parallel variable.
  If the variable is 1D (Shape []S), then a 1D FFT is performed.
  If the variable is 2D (Shape [][]S), then a 2D FFT is performed.
  Transformed data is returned in pv (in-place operation).

  CALLS:  cm_c_fft(), a CMF subroutine located in the 
            CMF module, CM_futil.fcm.
 **************************************************************/
{
  
  CMC_descriptor_t cmplx_desc;

      
  cmplx_desc = CMC_wrap_pvar(pv);
    
  CMC_CALL_FORTRAN(cm_c_fft_(cmplx_desc));

  CMC_free_desc(cmplx_desc);

}







